<template>
  <div class="mod-config">
    <basic-container>
      <ttable
        :table="table"
        @handleSelectionChange="handleSelectionChange"
        :uploadInfo="uploadInfo"
        :prelang="prelang"
        :options="options"
        :bottomOffset="200"
        :ajaxFun="ajaxFun"
        ref="scrapMainTable"
      >
        <template #toolbar></template>
      </ttable>
      <!-- 弹窗, 新增 / 修改 -->
      <table-form
        v-if="addOrUpdateVisible"
        ref="addOrUpdate"
        @refreshDataList="getData"
      ></table-form>
    </basic-container>
  </div>
</template>

<script>
import { delObj, fetchList } from '@/api/warehouse/scrapmain'
import TableForm from './scrapmain-form'
import ttable from '@/views/common/ztt-table.vue'
import { mapGetters } from 'vuex'
import { submitScrapDetail } from '../../../api/warehouse/scrapmain'
import { loadFactoryList } from '../../../api/basic/factory'

export default {
  data() {
    return {
      factoryOptions: [],
      ajaxFun: fetchList,
      multipleSelection: [],
      uploadInfo: {
        // 是否展示上传EXCEL以及对应的url
        isShow: false,
        url: ''
      },
      prelang: 'operation',
      options: {
        height: 300, // 默认高度-为了表头固定
        stripe: true, // 是否为斑马纹 table
        highlightCurrentRow: false, // 是否要高亮当前行
        border: true, // 是否有纵向边框
        lazy: false, // 是否需要懒加载
        fit: true, // 列的宽度是否自撑开
        multiSelect: true, //
        seqNo: true,
        isRefresh: true, // 是否显示刷新按钮
        isShowHide: true, // 是否显示显影按钮
        isSearch: false, // 高级查询按钮
        defaultOrderBy: { column: 'id', direction: 'desc' }
      },
      table: {
        total: 0,
        currentPage: 1,
        pageSize: 20,
        data: [],
        // 标题
        column: [
          // 报废单号
          {
            minWidth: '140',
            prop: 'scrapNo',
            label: '报废单号',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text',
            render: { fun: this.addOrUpdateHandle }
          },
          // 报废类型
          {
            minWidth: '120',
            prop: 'scrapType',
            label: '报废类型',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          // 报废描述
          {
            minWidth: '200',
            prop: 'scrapDesc',
            label: '报废描述',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          // 工厂
          {
            minWidth: '140',
            prop: 'factoryId',
            label: '工厂',
            sort: true,
            isTrue: true,
            isSearch: false,
            searchInfoType: 'text',
            formatter: this.getReserveFactory,
            optList: () => {
              return this.factoryOptions
            }
          },
          // 状态
          {
            minWidth: '120',
            prop: 'status',
            label: '状态',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          // 零件
          {
            minWidth: '120',
            prop: 'partNo',
            label: '零件',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          // 零件批号
          {
            minWidth: '200',
            prop: 'partBatchNo',
            label: '零件批号',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          // 零件序列号
          {
            minWidth: '140',
            prop: 'partSn',
            label: '零件序列号',
            sort: true,
            isTrue: true,
            isSearch: false,
            searchInfoType: 'text'
          },
          // 仓库
          {
            minWidth: '120',
            prop: 'warehouseName',
            label: '仓库',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          // 库位
          {
            minWidth: '140',
            prop: 'locationNo',
            label: '库位号',
            sort: true,
            isTrue: true,
            isSearch: false,
            searchInfoType: 'text'
          },
          // 库存数量
          {
            minWidth: '120',
            prop: 'locQty',
            label: '库存数量',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          // 报废数量
          {
            minWidth: '120',
            prop: 'scrapQty',
            label: '报废数量',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          {
            minWidth: '80',
            prop: 'unit',
            label: '零件单位',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          // 报废人
          {
            minWidth: '120',
            prop: 'scrapUser',
            label: '报废人',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'text'
          },
          // 报废时间
          {
            minWidth: '120',
            width: '200px',
            prop: 'scrapTime',
            label: '报废时间',
            sort: true,
            isTrue: true,
            isSearch: true,
            searchInfoType: 'datetimerange'
          }
        ],
        toolbar: [
          {
            text: '新增',
            type: 'primary',
            fun: this.addOrUpdateHandle,
            disabled: this.disabled,
            permitArr: []
          },
          {
            text: '提交',
            fun: this.submitScrapMain,
            disabled: this.disabled,
            permitArr: ['未下发']
          }
        ],
        operator: [
          {
            text: '删除',
            type: 'text',
            size: 'small',
            fun: this.deleteHandle,
            show: {
              val: ['未下发'],
              key: 'status'
            }
          }
        ],
        operatorConfig: {
          fixed: 'right',
          label: '操作',
          width: 200,
          minWidth: 100
        }
      },
      addOrUpdateVisible: false
    }
  },
  created() {
    this.initFactorySelect()
  },
  components: {
    ttable,
    TableForm
  },
  computed: {
    ...mapGetters(['permissions'])
  },
  methods: {
    // 获取数据列表
    getData() {
      this.$refs.scrapMainTable.getDataList()
    },
    handleSelectionChange(val) {
      // 根据状态，禁用表头按钮
      // 筛选出选中记录的状态
      var stateArr = val.map(function(value, index) {
        return value.status
      })
      // 选中状态数组元素去重
      var uniqueStateArr = []
      for (var i = 0; i < stateArr.length; i++) {
        if (uniqueStateArr.indexOf(stateArr[i]) == -1) {
          uniqueStateArr.push(stateArr[i])
        }
      }
      // 循环表头按钮，判断每个按钮的permitArr是否完全包含选中状态，若完全包含，则按钮亮，否则按钮灰
      var toolbar = this.table.toolbar
      for (var j = 0; j < toolbar.length; j++) {
        if (
          uniqueStateArr.every((val) =>
            toolbar[j].permitArr.length <= 0
              ? true
              : toolbar[j].permitArr.includes(val)
          )
        ) {
          toolbar[j].disabled = false
        } else {
          toolbar[j].disabled = true
        }
      }
      // 将选中记录赋值给multipleSelection
      this.multipleSelection = val
    },
    // 获取工厂的数据
    initFactorySelect() {
      loadFactoryList().then((res) => {
        this.factoryOptions = JSON.parse(
          JSON.stringify(res.data)
            .replace(/id/g, 'value')
            .replace(/factoryName/g, 'label')
        )
      })
    },
    getReserveFactory(row, column, cellValue) {
      this.factoryOptions.forEach((obj) => {
        if (obj.value == cellValue) {
          cellValue = obj.label
        }
      })
      return cellValue
    },
    // 下发
    submitScrapMain() {
      if (this.multipleSelection == 0) {
        this.$message.success('请选择盘点单号')
        return
      }
      submitScrapDetail(this.multipleSelection).then((response) => {
        this.$message.success('下发成功')
        this.getData()
      })
    },
    // 新增 / 修改
    addOrUpdateHandle(row) {
      this.addOrUpdateVisible = true
      this.$nextTick(() => {
        this.$refs.addOrUpdate.init(row == null ? null : row.id)
      })
    },
    // 删除
    deleteHandle(row) {
      this.$confirm('是否确认删除ID为' + row.id, '提示', {
        confirmButtonText: '确定',
        cancelButtonText: '取消',
        type: 'warning'
      })
        .then(function() {
          return delObj(row)
        })
        .then((data) => {
          this.$message.success('删除成功')
          this.getData()
        })
    }
  }
}
</script>
